@@ -1,5 +1,5 @@
 /* Darwin support for GDB, the GNU debugger.
-   Copyright (C) 1997-2013 Free Software Foundation, Inc.
+   Copyright (C) 1997-2021 Free Software Foundation, Inc.
 
    Contributed by Apple Computer, Inc.
 
@@ -27,14 +27,13 @@
 #include "objfiles.h"
 #include "gdbcmd.h"
 #include "regcache.h"
-#include "gdb_assert.h"
 #include "i386-tdep.h"
 #include "i387-tdep.h"
 #include "gdbarch.h"
 #include "arch-utils.h"
 #include "gdbcore.h"
 
-#include "i386-nat.h"
+#include "x86-nat.h"
 #include "darwin-nat.h"
 #include "i386-darwin-tdep.h"
 
@@ -44,16 +43,25 @@
 #include "amd64-darwin-tdep.h"
 #endif
 
+struct i386_darwin_nat_target final : public x86_nat_target<darwin_nat_target>
+{
+  /* Add our register access methods.  */
+  void fetch_registers (struct regcache *, int) override;
+  void store_registers (struct regcache *, int) override;
+};
+
+static struct i386_darwin_nat_target darwin_target;
+
 /* Read register values from the inferior process.
    If REGNO is -1, do this for all registers.
    Otherwise, REGNO specifies which register (so we can save time).  */
-static void
-i386_darwin_fetch_inferior_registers (struct target_ops *ops,
-				      struct regcache *regcache, int regno)
+
+void
+i386_darwin_nat_target::fetch_registers (struct regcache *regcache, int regno)
 {
-  thread_t current_thread = ptid_get_tid (inferior_ptid);
+  thread_t current_thread = regcache->ptid ().tid ();
   int fetched = 0;
-  struct gdbarch *gdbarch = get_regcache_arch (regcache);
+  struct gdbarch *gdbarch = regcache->arch ();
 
 #ifdef BFD64
   if (gdbarch_ptr_bit (gdbarch) == 64)
@@ -74,6 +82,11 @@ i386_darwin_fetch_inferior_registers (struct target_ops *ops,
 				 (unsigned long) current_thread);
 	      MACH_CHECK_ERROR (ret);
 	    }
+
+	  /* Some kernels don't sanitize the values.  */
+	  gp_regs.uts.ts64.__fs &= 0xffff;
+	  gp_regs.uts.ts64.__gs &= 0xffff;
+
 	  amd64_supply_native_gregset (regcache, &gp_regs.uts, -1);
           fetched++;
         }
@@ -119,9 +132,8 @@ i386_darwin_fetch_inferior_registers (struct target_ops *ops,
 	      MACH_CHECK_ERROR (ret);
 	    }
 	  for (i = 0; i < I386_NUM_GREGS; i++)
-	    regcache_raw_supply
-	      (regcache, i,
-	       (char *)&gp_regs + i386_darwin_thread_state_reg_offset[i]);
+	    regcache->raw_supply
+	      (i, (char *) &gp_regs + i386_darwin_thread_state_reg_offset[i]);
 
           fetched++;
         }
@@ -151,7 +163,7 @@ i386_darwin_fetch_inferior_registers (struct target_ops *ops,
   if (! fetched)
     {
       warning (_("unknown register %d"), regno);
-      regcache_raw_supply (regcache, regno, NULL);
+      regcache->raw_supply (regno, NULL);
     }
 }
 
@@ -159,12 +171,12 @@ i386_darwin_fetch_inferior_registers (struct target_ops *ops,
    If REGNO is -1, do this for all registers.
    Otherwise, REGNO specifies which register (so we can save time).  */
 
-static void
-i386_darwin_store_inferior_registers (struct target_ops *ops,
-				      struct regcache *regcache, int regno)
+void
+i386_darwin_nat_target::store_registers (struct regcache *regcache,
+					 int regno)
 {
-  thread_t current_thread = ptid_get_tid (inferior_ptid);
-  struct gdbarch *gdbarch = get_regcache_arch (regcache);
+  thread_t current_thread = regcache->ptid ().tid ();
+  struct gdbarch *gdbarch = regcache->arch ();
 
 #ifdef BFD64
   if (gdbarch_ptr_bit (gdbarch) == 64)
@@ -184,6 +196,10 @@ i386_darwin_store_inferior_registers (struct target_ops *ops,
 
 	  amd64_collect_native_gregset (regcache, &gp_regs.uts, regno);
 
+	  /* Some kernels don't sanitize the values.  */
+	  gp_regs.uts.ts64.__fs &= 0xffff;
+	  gp_regs.uts.ts64.__gs &= 0xffff;
+
           ret = thread_set_state (current_thread, x86_THREAD_STATE,
                                   (thread_state_t) &gp_regs,
                                   x86_THREAD_STATE_COUNT);
@@ -228,9 +244,8 @@ i386_darwin_store_inferior_registers (struct target_ops *ops,
 
 	  for (i = 0; i < I386_NUM_GREGS; i++)
 	    if (regno == -1 || regno == i)
-	      regcache_raw_collect
-		(regcache, i,
-		 (char *)&gp_regs + i386_darwin_thread_state_reg_offset[i]);
+	      regcache->raw_collect
+		(i, (char *) &gp_regs + i386_darwin_thread_state_reg_offset[i]);
 
           ret = thread_set_state (current_thread, x86_THREAD_STATE32,
                                   (thread_state_t) &gp_regs,
@@ -260,132 +275,178 @@ i386_darwin_store_inferior_registers (struct target_ops *ops,
     }
 }
 
-#ifdef HW_WATCHPOINT_NOT_YET_ENABLED
 /* Support for debug registers, boosted mostly from i386-linux-nat.c.  */
 
 static void
-i386_darwin_dr_set (int regnum, uint32_t value)
+i386_darwin_dr_set (int regnum, CORE_ADDR value)
 {
-  int current_pid;
   thread_t current_thread;
   x86_debug_state_t dr_regs;
   kern_return_t ret;
-  unsigned int dr_count = x86_DEBUG_STATE_COUNT;
+  unsigned int dr_count;
 
   gdb_assert (regnum >= 0 && regnum <= DR_CONTROL);
 
-  current_thread = ptid_get_tid (inferior_ptid);
+  current_thread = inferior_ptid.tid ();
 
-  dr_regs.dsh.flavor = x86_DEBUG_STATE32;
-  dr_regs.dsh.count = x86_DEBUG_STATE32_COUNT;
+  dr_regs.dsh.flavor = x86_DEBUG_STATE;
+  dr_regs.dsh.count = x86_DEBUG_STATE_COUNT;
   dr_count = x86_DEBUG_STATE_COUNT;
-  ret = thread_get_state (current_thread, x86_DEBUG_STATE, 
+  ret = thread_get_state (current_thread, x86_DEBUG_STATE,
                           (thread_state_t) &dr_regs, &dr_count);
+  MACH_CHECK_ERROR (ret);
 
-  if (ret != KERN_SUCCESS)
+  switch (dr_regs.dsh.flavor)
     {
-      printf_unfiltered (_("Error reading debug registers "
-			   "thread 0x%x via thread_get_state\n"),
-			 (int) current_thread);
-      MACH_CHECK_ERROR (ret);
+    case x86_DEBUG_STATE32:
+      switch (regnum)
+	{
+	case 0:
+	  dr_regs.uds.ds32.__dr0 = value;
+	  break;
+	case 1:
+	  dr_regs.uds.ds32.__dr1 = value;
+	  break;
+	case 2:
+	  dr_regs.uds.ds32.__dr2 = value;
+	  break;
+	case 3:
+	  dr_regs.uds.ds32.__dr3 = value;
+	  break;
+	case 4:
+	  dr_regs.uds.ds32.__dr4 = value;
+	  break;
+	case 5:
+	  dr_regs.uds.ds32.__dr5 = value;
+	  break;
+	case 6:
+	  dr_regs.uds.ds32.__dr6 = value;
+	  break;
+	case 7:
+	  dr_regs.uds.ds32.__dr7 = value;
+	  break;
+	}
+      break;
+#ifdef BFD64
+    case x86_DEBUG_STATE64:
+      switch (regnum)
+	{
+	case 0:
+	  dr_regs.uds.ds64.__dr0 = value;
+	  break;
+	case 1:
+	  dr_regs.uds.ds64.__dr1 = value;
+	  break;
+	case 2:
+	  dr_regs.uds.ds64.__dr2 = value;
+	  break;
+	case 3:
+	  dr_regs.uds.ds64.__dr3 = value;
+	  break;
+	case 4:
+	  dr_regs.uds.ds64.__dr4 = value;
+	  break;
+	case 5:
+	  dr_regs.uds.ds64.__dr5 = value;
+	  break;
+	case 6:
+	  dr_regs.uds.ds64.__dr6 = value;
+	  break;
+	case 7:
+	  dr_regs.uds.ds64.__dr7 = value;
+	  break;
+	}
+      break;
+#endif
     }
 
-  switch (regnum) 
-    {
-      case 0:
-        dr_regs.uds.ds32.__dr0 = value;
-        break;
-      case 1:
-        dr_regs.uds.ds32.__dr1 = value;
-        break;
-      case 2:
-        dr_regs.uds.ds32.__dr2 = value;
-        break;
-      case 3:
-        dr_regs.uds.ds32.__dr3 = value;
-        break;
-      case 4:
-        dr_regs.uds.ds32.__dr4 = value;
-        break;
-      case 5:
-        dr_regs.uds.ds32.__dr5 = value;
-        break;
-      case 6:
-        dr_regs.uds.ds32.__dr6 = value;
-        break;
-      case 7:
-        dr_regs.uds.ds32.__dr7 = value;
-        break;
-    }
+  ret = thread_set_state (current_thread, dr_regs.dsh.flavor,
+                          (thread_state_t) &dr_regs.uds, dr_count);
 
-  ret = thread_set_state (current_thread, x86_DEBUG_STATE, 
-                          (thread_state_t) &dr_regs, dr_count);
-
-  if (ret != KERN_SUCCESS)
-    {
-      printf_unfiltered (_("Error writing debug registers "
-			   "thread 0x%x via thread_get_state\n"),
-			 (int) current_thread);
-      MACH_CHECK_ERROR (ret);
-    }
+  MACH_CHECK_ERROR (ret);
 }
 
-static uint32_t
+static CORE_ADDR
 i386_darwin_dr_get (int regnum)
 {
   thread_t current_thread;
   x86_debug_state_t dr_regs;
   kern_return_t ret;
-  unsigned int dr_count = x86_DEBUG_STATE_COUNT;
+  unsigned int dr_count;
 
   gdb_assert (regnum >= 0 && regnum <= DR_CONTROL);
 
-  current_thread = ptid_get_tid (inferior_ptid);
+  current_thread = inferior_ptid.tid ();
 
-  dr_regs.dsh.flavor = x86_DEBUG_STATE32;
-  dr_regs.dsh.count = x86_DEBUG_STATE32_COUNT;
+  dr_regs.dsh.flavor = x86_DEBUG_STATE;
+  dr_regs.dsh.count = x86_DEBUG_STATE_COUNT;
   dr_count = x86_DEBUG_STATE_COUNT;
-  ret = thread_get_state (current_thread, x86_DEBUG_STATE, 
+  ret = thread_get_state (current_thread, x86_DEBUG_STATE,
                           (thread_state_t) &dr_regs, &dr_count);
+  MACH_CHECK_ERROR (ret);
 
-  if (ret != KERN_SUCCESS)
+  switch (dr_regs.dsh.flavor)
     {
-      printf_unfiltered (_("Error reading debug registers "
-			   "thread 0x%x via thread_get_state\n"),
-			 (int) current_thread);
-      MACH_CHECK_ERROR (ret);
-    }
-
-  switch (regnum) 
-    {
-      case 0:
-        return dr_regs.uds.ds32.__dr0;
-      case 1:
-        return dr_regs.uds.ds32.__dr1;
-      case 2:
-        return dr_regs.uds.ds32.__dr2;
-      case 3:
-        return dr_regs.uds.ds32.__dr3;
-      case 4:
-        return dr_regs.uds.ds32.__dr4;
-      case 5:
-        return dr_regs.uds.ds32.__dr5;
-      case 6:
-        return dr_regs.uds.ds32.__dr6;
-      case 7:
-        return dr_regs.uds.ds32.__dr7;
-      default:
-        return -1;
+    case x86_DEBUG_STATE32:
+      switch (regnum)
+	{
+	case 0:
+	  return dr_regs.uds.ds32.__dr0;
+	case 1:
+	  return dr_regs.uds.ds32.__dr1;
+	case 2:
+	  return dr_regs.uds.ds32.__dr2;
+	case 3:
+	  return dr_regs.uds.ds32.__dr3;
+	case 4:
+	  return dr_regs.uds.ds32.__dr4;
+	case 5:
+	  return dr_regs.uds.ds32.__dr5;
+	case 6:
+	  return dr_regs.uds.ds32.__dr6;
+	case 7:
+	  return dr_regs.uds.ds32.__dr7;
+	default:
+	  return -1;
+	}
+      break;
+#ifdef BFD64
+    case x86_DEBUG_STATE64:
+      switch (regnum)
+	{
+	case 0:
+	  return dr_regs.uds.ds64.__dr0;
+	case 1:
+	  return dr_regs.uds.ds64.__dr1;
+	case 2:
+	  return dr_regs.uds.ds64.__dr2;
+	case 3:
+	  return dr_regs.uds.ds64.__dr3;
+	case 4:
+	  return dr_regs.uds.ds64.__dr4;
+	case 5:
+	  return dr_regs.uds.ds64.__dr5;
+	case 6:
+	  return dr_regs.uds.ds64.__dr6;
+	case 7:
+	  return dr_regs.uds.ds64.__dr7;
+	default:
+	  return -1;
+	}
+      break;
+#endif
+    default:
+      return -1;
     }
 }
 
-void
+static void
 i386_darwin_dr_set_control (unsigned long control)
 {
   i386_darwin_dr_set (DR_CONTROL, control);
 }
 
-void
+static void
 i386_darwin_dr_set_addr (int regnum, CORE_ADDR addr)
 {
   gdb_assert (regnum >= 0 && regnum <= DR_LASTADDR - DR_FIRSTADDR);
@@ -393,24 +454,23 @@ i386_darwin_dr_set_addr (int regnum, CORE_ADDR addr)
   i386_darwin_dr_set (DR_FIRSTADDR + regnum, addr);
 }
 
-CORE_ADDR
+static CORE_ADDR
 i386_darwin_dr_get_addr (int regnum)
 {
   return i386_darwin_dr_get (regnum);
 }
 
-unsigned long
+static unsigned long
 i386_darwin_dr_get_status (void)
 {
   return i386_darwin_dr_get (DR_STATUS);
 }
 
-unsigned long
+static unsigned long
 i386_darwin_dr_get_control (void)
 {
   return i386_darwin_dr_get (DR_CONTROL);
 }
-#endif
 
 void
 darwin_check_osabi (darwin_inferior *inf, thread_t thread)
@@ -439,7 +499,7 @@ darwin_check_osabi (darwin_inferior *inf, thread_t thread)
 	info.bfd_arch_info = bfd_lookup_arch (bfd_arch_i386,
 					      bfd_mach_x86_64);
       else
-	info.bfd_arch_info = bfd_lookup_arch (bfd_arch_i386, 
+	info.bfd_arch_info = bfd_lookup_arch (bfd_arch_i386,
 					      bfd_mach_i386_i386);
       gdbarch_update_p (info);
     }
@@ -539,14 +599,14 @@ darwin_set_sstep (thread_t thread, int enable)
     case x86_THREAD_STATE32:
       {
 	__uint32_t bit = enable ? X86_EFLAGS_T : 0;
-	
+
 	if (enable && i386_darwin_sstep_at_sigreturn (&regs))
 	  return;
 	if ((regs.uts.ts32.__eflags & X86_EFLAGS_T) == bit)
 	  return;
 	regs.uts.ts32.__eflags
 	  = (regs.uts.ts32.__eflags & ~X86_EFLAGS_T) | bit;
-	kret = thread_set_state (thread, x86_THREAD_STATE, 
+	kret = thread_set_state (thread, x86_THREAD_STATE,
 				 (thread_state_t) &regs, count);
 	MACH_CHECK_ERROR (kret);
       }
@@ -562,7 +622,7 @@ darwin_set_sstep (thread_t thread, int enable)
 	  return;
 	regs.uts.ts64.__rflags
 	  = (regs.uts.ts64.__rflags & ~X86_EFLAGS_T) | bit;
-	kret = thread_set_state (thread, x86_THREAD_STATE, 
+	kret = thread_set_state (thread, x86_THREAD_STATE,
 				 (thread_state_t) &regs, count);
 	MACH_CHECK_ERROR (kret);
       }
@@ -573,8 +633,9 @@ darwin_set_sstep (thread_t thread, int enable)
     }
 }
 
+void _initialize_i386_darwin_nat ();
 void
-darwin_complete_target (struct target_ops *target)
+_initialize_i386_darwin_nat ()
 {
 #ifdef BFD64
   amd64_native_gregset64_reg_offset = amd64_darwin_thread_state_reg_offset;
@@ -583,6 +644,18 @@ darwin_complete_target (struct target_ops *target)
   amd64_native_gregset32_num_regs = i386_darwin_thread_state_num_regs;
 #endif
 
-  target->to_fetch_registers = i386_darwin_fetch_inferior_registers;
-  target->to_store_registers = i386_darwin_store_inferior_registers;
+  x86_dr_low.set_control = i386_darwin_dr_set_control;
+  x86_dr_low.set_addr = i386_darwin_dr_set_addr;
+  x86_dr_low.get_addr = i386_darwin_dr_get_addr;
+  x86_dr_low.get_status = i386_darwin_dr_get_status;
+  x86_dr_low.get_control = i386_darwin_dr_get_control;
+
+  /* Let's assume that the kernel is 64 bits iff the executable is.  */
+#ifdef __x86_64__
+  x86_set_debug_register_length (8);
+#else
+  x86_set_debug_register_length (4);
+#endif
+
+  add_inf_child_target (&darwin_target);
 }
