@@ -1,5 +1,5 @@
 /* Interface to the inferior event handling code for GDB, the GNU debugger.
-   Copyright (C) 1999-2013 Free Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
    Written by Elena Zannoni <ezannoni@cygnus.com> of Cygnus Solutions.
 
    This file is part of GDB.
@@ -20,7 +20,8 @@
 #ifndef INF_LOOP_H
 #define INF_LOOP_H
 
-extern void inferior_event_handler (enum inferior_event_type event_type, 
-				    void* client_data);
+#include "target.h"             /* For enum inferior_event_type.  */
+
+extern void inferior_event_handler (enum inferior_event_type event_type);
 
 #endif /* #ifndef INF_LOOP_H */
