@@ -1,6 +1,6 @@
 /* This testcase is part of GDB, the GNU debugger.
 
-   Copyright 2009-2013 Free Software Foundation, Inc.
+   Copyright 2009-2021 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -37,8 +37,26 @@ int verylongfun()
   glob *= 8;
   glob += 9;
   glob *= 9;
+  return 0;
 }
 
+void
+mailand()
+{
+  glob = 46;
+}
+
+int
+foo(int x) {
+  return x + 92;
+}
+
+void
+goodbye() {
+  ++glob;
+}
+
+int
 main() {
   mailand();
   foo(glob);
@@ -46,17 +64,4 @@ main() {
   goodbye();
 }
 
-foo(int x) {
-  return x + 92;
-}
-
-mailand()
-{
-  glob = 46;
-}
-
 void commonfun() { mailand(); } /* from goodbye */
-
-goodbye() {
-  ++glob;
-}
