@@ -93,7 +93,7 @@ struct command_table_entry linux_command_table[] = {
 	{"mach",    cmd_mach,    help_mach,    0},
 	{"map",     cmd_map,     help_map,     HIDDEN_COMMAND},
 	{"mod",     cmd_mod,     help_mod,     0},
-	{"mount",   cmd_mount,   help_mount,   0},
+	{"mount",   cmd_mount,   help_mount,   REFRESH_TASK_TABLE},
 	{"net",	    cmd_net,	help_net,      REFRESH_TASK_TABLE},
 	{"p",       cmd_p,       help_p,       0},
 	{"ps",      cmd_ps,      help_ps,      REFRESH_TASK_TABLE},
@@ -105,6 +105,7 @@ struct command_table_entry linux_command_table[] = {
         {"rd",      cmd_rd,      help_rd,      MINIMAL},
 	{"repeat",  cmd_repeat,  help_repeat,  0},
 	{"runq",    cmd_runq,    help_runq,    REFRESH_TASK_TABLE},
+	{"sbitmapq", cmd_sbitmapq, help_sbitmapq, 0},
         {"search",  cmd_search,  help_search,  0},
         {"set",     cmd_set,     help_set,     REFRESH_TASK_TABLE | MINIMAL},
         {"sig",     cmd_sig,     help_sig,     REFRESH_TASK_TABLE},
