@@ -183,7 +183,7 @@ s390_dump_machdep_table(ulong arg)
 	fprintf(fp, ")\n");
 
 	fprintf(fp, "             kvbase: %lx\n", machdep->kvbase);
-	fprintf(fp, "  identity_map_base: %lx\n", machdep->kvbase);
+	fprintf(fp, "  identity_map_base: %lx\n", machdep->identity_map_base);
 	fprintf(fp, "           pagesize: %d\n", machdep->pagesize);
 	fprintf(fp, "          pageshift: %d\n", machdep->pageshift);
 	fprintf(fp, "           pagemask: %llx\n", machdep->pagemask);
@@ -442,6 +442,9 @@ s390_verify_symbol(const char *name, ulong value, char type)
 	if (strstr(name, "L2\002") == name)
 	    	return FALSE;
 
+	if (STREQ(name, ".rodata"))
+		return TRUE;
+
 	/* throw away all symbols containing a '.' */
 	for(i = 0; i < strlen(name);i++){
 		if(name[i] == '.')
